--- THOMAS96-MODDING
--- 08/08/2023
--- MowerEffectExtension.lua
--- Copyright (c) THOMAS96-MODDING

MowerEffectExtension = {}
MowerEffectExtension.MOD_NAME = g_currentModName


function MowerEffectExtension.initSpecialization()
   local schema = Vehicle.xmlSchema
   schema:setXMLSpecializationType("MowerEffectExtension")
   EffectManager.registerEffectXMLPaths(schema, "vehicle.mowerEffectExtension.effect")
   schema:setXMLSpecializationType()
end



function MowerEffectExtension.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(WorkArea, specializations) 
	    and SpecializationUtil.hasSpecialization(TestAreas, specializations)
end

function MowerEffectExtension.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "spec_mowerEffectExtension", MowerEffectExtension.spec_mowerEffectExtension)
end

function MowerEffectExtension.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", MowerEffectExtension)
    SpecializationUtil.registerEventListener(vehicleType, "onDelete", MowerEffectExtension)
    SpecializationUtil.registerEventListener(vehicleType, "onReadUpdateStream", MowerEffectExtension)
    SpecializationUtil.registerEventListener(vehicleType, "onWriteUpdateStream", MowerEffectExtension)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", MowerEffectExtension)
end

function MowerEffectExtension:spec_mowerEffectExtension()
    return self[SPEC_NAME];
end


function MowerEffectExtension:onLoad(savegame)
    self.spec_mowerEffectExtension = self[("spec_%s.mowerEffectExtension"):format(MowerEffectExtension.MOD_NAME)]
    local spec = self.spec_mowerEffectExtension

    if self.isClient then
        spec.cutterEffects = g_effectManager:loadEffect(self.xmlFile, "vehicle.mowerEffectExtension.effect", self.components, self, self.i3dMappings)
    end

    spec.effectDirtyFlag = self:getNextDirtyFlag()

    spec.effectsAreRunning = false
    spec.lastFruitTypeIndex = FruitType.UNKNOWN
    spec.lastFruitGrowthState = 0

    spec.raycastsValid = true
    spec.lastSmoothSpeed = 0

    self:setTestAreaRequirements(FruitType.GRASS, nil, false)
end




function MowerEffectExtension:onDelete()
    local spec = self.spec_mowerEffectExtension
    g_effectManager:deleteEffects(spec.cutterEffects)
end


function MowerEffectExtension:onReadUpdateStream(streamId, timestamp, connection)
    if connection:getIsServer() then
        local spec = self.spec_mowerEffectExtension

        local effectsAreRunning = streamReadBool(streamId)
        if effectsAreRunning then
            spec.lastFruitGrowthState = streamReadUIntN(streamId, 4)
            spec.lastFruitTypeIndex = streamReadUIntN(streamId, FruitTypeManager.SEND_NUM_BITS)
        end

        if effectsAreRunning ~= spec.effectsAreRunning then
            spec.effectsAreRunning = effectsAreRunning

            if not effectsAreRunning then
                g_effectManager:stopEffects(spec.cutterEffects)
            end
        end
    end
end


function MowerEffectExtension:onWriteUpdateStream(streamId, connection, dirtyMask)
    if not connection:getIsServer() then
        local spec = self.spec_mowerEffectExtension
        if streamWriteBool(streamId, spec.effectsAreRunning) then
            streamWriteUIntN(streamId, spec.lastFruitGrowthState, 4)
            streamWriteUIntN(streamId, spec.lastFruitTypeIndex, FruitTypeManager.SEND_NUM_BITS)
        end
    end
end


function MowerEffectExtension:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
    local spec = self.spec_mowerEffectExtension

    if self:getIsTurnedOn() and self:getLastSpeed() > 0.5 then
        local currentTestAreaMinX, currentTestAreaMaxX, testAreaMinX, testAreaMaxX = self:getTestAreaWidthByWorkAreaIndex(1)

        local reset = false
        if currentTestAreaMinX == -math.huge and currentTestAreaMaxX == math.huge then
            currentTestAreaMinX = 0
            currentTestAreaMaxX = 0
            reset = true
        end

        if spec.movingDirection > 0 then
            currentTestAreaMinX = currentTestAreaMinX * -1
            currentTestAreaMaxX = currentTestAreaMaxX * -1
            if currentTestAreaMaxX < currentTestAreaMinX then
                local t = currentTestAreaMinX
                currentTestAreaMinX = currentTestAreaMaxX
                currentTestAreaMaxX = t
            end
        end

        local inputFruitType, inputGrowthState
        local isActive
        if self.isServer then
            inputFruitType, inputGrowthState = FruitType.UNKNOWN, 3
            if self.spec_mower ~= nil then
                local specMower = self.spec_mower
                if g_time - specMower.workAreaParameters.lastCutTime < 500 then
                    inputFruitType = specMower.workAreaParameters.lastInputFruitType
                    inputGrowthState = specMower.workAreaParameters.lastInputGrowthState
                end
            end

            isActive = not reset and inputFruitType ~= nil and inputFruitType ~= FruitType.UNKNOWN

            if isActive then
                if not spec.effectsAreRunning then
                    spec.effectsAreRunning = true
                    self:raiseDirtyFlags(spec.effectDirtyFlag)
                end
            else
                if spec.effectsAreRunning then
                    g_effectManager:stopEffects(spec.cutterEffects)
                    spec.effectsAreRunning = false
                    self:raiseDirtyFlags(spec.effectDirtyFlag)
                end
            end

            spec.lastFruitTypeIndex, spec.lastFruitGrowthState = inputFruitType, inputGrowthState
        else
            inputFruitType, inputGrowthState = spec.lastFruitTypeIndex, spec.lastFruitGrowthState
            isActive = spec.effectsAreRunning and spec.lastFruitTypeIndex ~= FruitType.UNKNOWN
        end

        if isActive then
            g_effectManager:setFruitType(spec.cutterEffects, inputFruitType, inputGrowthState)
            g_effectManager:setMinMaxWidth(spec.cutterEffects, currentTestAreaMinX, currentTestAreaMaxX, currentTestAreaMinX / testAreaMinX, currentTestAreaMaxX / testAreaMaxX, reset)
            g_effectManager:startEffects(spec.cutterEffects)
        end
    else
        if spec.effectsAreRunning then
            g_effectManager:stopEffects(spec.cutterEffects)
            spec.effectsAreRunning = false
            self:raiseDirtyFlags(spec.effectDirtyFlag)
        end
    end

    local lastSpeed = self.lastSignedSpeed * 1000.0
end